@@ -163,7 +163,6 @@ from rdflib.syntax.NamespaceManager import NamespaceManager
 from rdflib import sparql
 from rdflib.QueryResult import QueryResult
 from rdflib.URLInputSource import URLInputSource
-
 from xml.sax.xmlreader import InputSource
 from xml.sax.saxutils import prepare_input_source
 
@@ -179,6 +178,19 @@ try:
 except ImportError:
     from md5 import md5    
 
+def describe(terms,bindings,graph):
+    """ 
+    Default DESCRIBE returns all incomming and outgoing statements about the given terms 
+    """
+    from rdflib.sparql.sparqlOperators import getValue
+    g=Graph()
+    terms=[getValue(i)(bindings) for i in terms]
+    for s,p,o in graph.triples_choices((terms,None,None)):
+        g.add((s,p,o))
+    for s,p,o in graph.triples_choices((None,None,terms)):
+        g.add((s,p,o))
+    return g
+
 class Graph(Node):
     """An RDF Graph
 
@@ -553,6 +565,46 @@ class Graph(Node):
                 yield item
             list = self.value(list, RDF.rest)
 
+    def transitiveClosure(self,func,arg):
+        """
+        Generates transitive closure of a user-defined 
+        function against the graph
+        
+        >>> from rdflib.Collection import Collection
+        >>> g=Graph()
+        >>> a=BNode('foo')
+        >>> b=BNode('bar')
+        >>> c=BNode('baz')
+        >>> g.add((a,RDF.first,RDF.type))
+        >>> g.add((a,RDF.rest,b))
+        >>> g.add((b,RDF.first,RDFS.label))
+        >>> g.add((b,RDF.rest,c))
+        >>> g.add((c,RDF.first,RDFS.comment))
+        >>> g.add((c,RDF.rest,RDF.nil))
+        >>> def topList(node,g):
+        ...    for s in g.subjects(RDF.rest,node):
+        ...       yield s
+        >>> def reverseList(node,g):
+        ...    for f in g.objects(node,RDF.first):
+        ...       print f
+        ...    for s in g.subjects(RDF.rest,node):
+        ...       yield s
+        
+        >>> [rt for rt in g.transitiveClosure(topList,RDF.nil)]
+        [rdflib.BNode('baz'), rdflib.BNode('bar'), rdflib.BNode('foo')]
+        
+        >>> [rt for rt in g.transitiveClosure(reverseList,RDF.nil)]
+        http://www.w3.org/2000/01/rdf-schema#comment
+        http://www.w3.org/2000/01/rdf-schema#label
+        http://www.w3.org/1999/02/22-rdf-syntax-ns#type
+        [rdflib.BNode('baz'), rdflib.BNode('bar'), rdflib.BNode('foo')]
+        
+        """
+        for rt in func(arg,self):
+            yield rt
+            for rt_2 in self.transitiveClosure(func,rt):
+                yield rt_2
+
     def transitive_objects(self, subject, property, remember=None):
         """Transitively generate objects for the `property` relationship
 
@@ -665,7 +717,9 @@ class Graph(Node):
         self.parse(source, publicID, format)
 
     def query(self, strOrQuery, initBindings={}, initNs={}, DEBUG=False,
-              processor="sparql"):
+              dataSetBase=None,
+              processor="sparql",
+              extensionFunctions={sparql.DESCRIBE:describe}):
         """
         Executes a SPARQL query (eventually will support Versa queries with same method) against this Graph
         strOrQuery - Is either a string consisting of the SPARQL query or an instance of rdflib.sparql.bison.Query.Query
@@ -676,12 +730,21 @@ class Graph(Node):
         """
         assert processor == 'sparql',"SPARQL is currently the only supported RDF query language"
         p = plugin.get(processor, sparql.Processor)(self)
-        return plugin.get('SPARQLQueryResult',QueryResult)(p.query(strOrQuery, initBindings, initNs, DEBUG))
+        return plugin.get('SPARQLQueryResult',QueryResult)(p.query(strOrQuery,
+                                                                   initBindings,
+                                                                   initNs, 
+                                                                   DEBUG, 
+                                                                   dataSetBase,
+                                                                   extensionFunctions))
 
         processor_plugin = plugin.get(processor, sparql.Processor)(self.store)
         qresult_plugin = plugin.get('SPARQLQueryResult', QueryResult)
 
-        res = processor_plugin.query(strOrQuery, initBindings, initNs, DEBUG)
+        res = processor_plugin.query(strOrQuery, 
+                                     initBindings, 
+                                     initNs, 
+                                     DEBUG, 
+                                     extensionFunctions=extensionFunctions)
         return qresult_plugin(res)
 
     def n3(self):
@@ -745,7 +808,6 @@ class Graph(Node):
         allNodes = obj.union(set(self.subjects()))
         return allNodes
 
-
 class ConjunctiveGraph(Graph):
 
     def __init__(self, store='default', identifier=None):
