@@ -1,7 +1,7 @@
 # RDF Library
 
-__version__ = "2.4.1"
-__date__ = "not/yet/released"
+__version__ = "2.4.2"
+__date__ = "2009/05/18"
 
 import sys
 # generator expressions require 2.4
@@ -19,14 +19,14 @@ from rdflib.Variable import Variable
 
 from rdflib.Namespace import Namespace
 
+from rdflib import RDF
+from rdflib import RDFS
+
 # from rdflib.Graph import Graph # perhaps in 3.0, but for 2.x we
 # don't want to break compatibility.
 from rdflib.Graph import BackwardCompatGraph as Graph
 from rdflib.Graph import ConjunctiveGraph
 
-from rdflib import RDF
-from rdflib import RDFS
-
 from rdflib.FileInputSource import FileInputSource
 from rdflib.URLInputSource import URLInputSource
 from rdflib.StringInputSource import StringInputSource
