@@ -97,6 +97,9 @@ register('BerkeleyDB', Store,
 register('BDBOptimized', Store,
          'rdflib.store.BDBOptimized', 'BDBOptimized')
 
+register('PostgreSQL', Store,
+         'rdflib.store.PostgreSQL', 'PostgreSQL')
+
 register('MySQL', Store,
          'rdflib.store.MySQL', 'MySQL')
 
