@@ -1,12 +1,10 @@
 import types, sets
-
-from rdflib import URIRef, BNode, Literal
+from pprint import pprint
+from rdflib import URIRef, BNode, Literal, Variable, RDF
+from rdflib.Graph import Graph, ConjunctiveGraph, ReadOnlyGraphAggregate
 from rdflib.Identifier import Identifier
-
 from rdflib.util import check_subject, list2set
-
 from rdflib.sparql import SPARQLError
-from rdflib.sparql.Unbound import Unbound
 from rdflib.sparql.sparqlGraph import SPARQLGraph
 from rdflib.sparql.graphPattern import GraphPattern
 
@@ -68,15 +66,15 @@ def _variablesToArray(variables,name='') :
             return None
         else :
             return [variables]
-    elif isinstance(variables,Unbound) :
-        return [variables.name]
+    elif isinstance(variables,Variable) :
+        return [variables]
     elif type(variables) == list or type(variables) == tuple :
         retval = []
         for s in variables :
             if isinstance(s,basestring) :
                 retval.append(s)
-            elif isinstance(s,Unbound) :
-                retval.append(s.name)
+            elif isinstance(s,Variable) :
+                retval.append(s)
             else :
                 raise SPARQLError("illegal type in '%s'; must be a string, unicode, or a Variable" % name)
     else :
@@ -94,9 +92,48 @@ def _createInitialBindings(pattern) :
     for c in pattern.unbounds :
         bindings[c] = None
     return bindings
+                       
+def _ancestorTraversal(node,selected=False):
+    if selected:
+        yield node
+    if node.parent:
+        for i in _ancestorTraversal(node.parent,selected=True):
+            yield i
+                        
+def _fetchBoundLeaves(node,previousBind=False,proxyTree=False):
+    """
+    Takes a SPARQLNode and returns a generator
+    over its bound leaves (including OPTIONAL proxies)
+    """
+    isaProxyTree = proxyTree or node.priorLeftJoin
+    if len(node.children) == 0 :
+        if node.bound and not node.clash:
+            #An OPTIONAL proxy is an expansion descendant which was 
+            #bound and valid (compatible) at a prior point and thus
+            #serves as the cumulative context for all subsequent operations
+            proxy=False
+            for optChild in reduce(lambda x,y: x+y,[list(_fetchBoundLeaves(o,previousBind,isaProxyTree))
+                                                for o in node.optionalTrees],[]):
+                proxy=True
+                yield optChild
+            if not proxy:
+                yield node
+        elif node.clash and previousBind and isaProxyTree:
+            #prior evaluation of LeftJoin was successful but later became
+            #excluded.  Note, this should not provide any bindings
+            yield node
+    else :
+        for c in node.children :
+            for proxy in _fetchBoundLeaves(c,previousBind,isaProxyTree):
+                yield proxy
 
+def isGroundQuad(quad):
+    for term in quad:
+        if isinstance(term,Variable):
+            return False
+    return True
 
-class _SPARQLNode:
+class _SPARQLNode(object):
     """
     The SPARQL implementation is based on the creation of a tree, each
     level for each statement in the 'where' clause of SPARQL.
@@ -147,7 +184,21 @@ class _SPARQLNode:
     @ivar optionalTrees: expansion trees for optional statements
     @type optionalTrees: array of _SPARQLNode instances
     """
-    def __init__(self,parent,bindings,statements,tripleStore) :
+    
+    __slots__ = ("expr",
+                 "tripleStore",
+                 "bindings",
+                 "optionalTrees",
+                 "bound",
+                 "clash",
+                 "priorLeftJoin",
+                 "dontSpawn",
+                 "children",
+                 "parent",
+                 "statement",
+                 "rest")
+    
+    def __init__(self,parent,bindings,statements,tripleStore,expr=None) :
         """
         @param parent:     parent node
         @param bindings:   a dictionary with the bindings that are already done or with None value if no binding yet
@@ -157,9 +208,12 @@ class _SPARQLNode:
         @param tripleStore: the 'owner' triple store
         @type tripleStore: L{sparqlGraph<rdflib.sparql.sparqlGraph.sparqlGraph>}
         """
+        self.priorLeftJoin       = False
+        self.expr = expr
         self.tripleStore         = tripleStore
         self.bindings            = bindings
         self.optionalTrees       = []
+        self.dontSpawn           = False
         if None in bindings.values() :
             self.bound = False
         else :
@@ -176,6 +230,67 @@ class _SPARQLNode:
             self.statement = None
             self.rest      = None
 
+    def __reduce__(self):
+        if self.statement:
+            statements = [self.statement]+ self.rest
+        else:
+            statements=[]
+        return (_SPARQLNode,
+                (self.parent,
+                 self.bindings,
+                 statements,
+                 self.tripleStore,
+                 self.expr),
+                 self.__getstate__())
+
+    def __getstate__(self):
+        if self.statement:
+            statements = [self.statement]+ self.rest
+        else:
+            statements=[]        
+        return (self.clash,
+                self.optionalTrees,
+                self.children,
+                self.parent,
+                self.bindings,
+                statements,
+                self.tripleStore,
+                self.expr,
+                self.bound,
+                self.priorLeftJoin,
+                self.dontSpawn)
+
+    def __setstate__(self, arg):
+        clash,optionals,children,parent,bindings,statements,tripleStore,expr,bound,plj,spawn = arg
+        #clash,optionals,children,proxy,spawn = arg
+        self.priorLeftJoin = plj
+        self.dontSpawn=spawn
+        self.bound=bound
+        self.clash=clash
+        self.optionalTrees=optionals
+        self.children=children
+        self.parent=parent
+        self.bindings=bindings
+        if len(statements) > 0 :
+            self.statement = statements[0]
+            self.rest      = statements[1:]
+        else :
+            self.statement = None
+            self.rest      = None
+        self.tripleStore=tripleStore
+        self.expr=expr        
+
+    def __repr__(self):
+        return "<SPARQLNode %s. %s children, %s OPTs.  clash: %s. bound: %s>"%(id(self),
+                                                                             len(self.children),
+                                                                             len(self.optionalTrees),
+                                                                             self.clash,
+                                                                             [k for k in self.bindings.keys()
+                                                                                 if self.bindings[k] is not None])
+        
+    def setupGraph(self,store):
+        self.tripleStore.setupGraph(store)
+
     def returnResult(self,select) :
         """
         Collect the result by search the leaves of the the tree. The
@@ -202,61 +317,41 @@ class _SPARQLNode:
         else :
             retval = []
             if self.bound == True and self.clash == False :
-                # This node should be able to contribute to the final results:
+                # This node should be able to contribute to the final results
+                # if it doesn't have any OPTIONAL proxies:
                 result = {}
+                #Determine if this node has an OPTIONAL 'proxy'
+                proxies = []
+                if self.optionalTrees:
+                    proxies = reduce(lambda x,y: x+y,[list(_fetchBoundLeaves(o))
+                                                for o in self.optionalTrees],[])
                 # This where the essential happens: the binding values are used to construct the selection result
-                if select :
-                    for a in select :
-                        if a in self.bindings :
-                            result[a] = self.bindings[a]
-                else :
-                    result = self.bindings.copy()
-                # Initial return block. If there is no optional processing, that is the result, in fact,
-                # because the for cycle below will not happen
-                retval = [result]
-                # The following remark in the SPARQL document is important at this point:
-                # "If a new variable is mentioned in an optional block (as mbox and hpage are mentioned
-                #  in the previous example), that variable can be mentioned in that block and can not be
-                #  mentioned in a subsequent block."
-                # What this means is that the various optional blocks do not interefere at this point
-                # and there is no need for a check whether a binding in a subsequent block
-                # clashes with an earlier optional block.
-                # The API checks this at the start.
-                # What happens here is that the result of the optional expantion is added to what is already
-                # there. Note that this may lead to a duplication of the result so far, if there are several
-                # alternatives returned by the optionals!
-                for optTree in self.optionalTrees :
-                    # get the results from the optional Tree...
-                    optionals = optTree.returnResult(select)
-                    # ... and extend the results accumulated so far with the new bindings
-                    # It is worth separating the case when there is only one optional block; it avoids
-                    # unnecessary copying
-                    if len(optionals) == 0 :
-                        # no contribution at all :-(
-                        continue
-                    elif len(optionals) == 1 :
-                        optResult = optionals[0]
-                        for res in retval :
-                            for k in optResult :
-                                if optResult[k] != None :
-                                    res[k] = optResult[k]
+                # sparql-p fix: A node with valid optional expansion trees should not
+                # contribute to bindings (the OPTIONAL expansion trees already account 
+                # for it's bindings)
+                # see: http://chatlogs.planetrdf.com/swig/2007-06-07.html#T19-28-43
+                if not proxies:
+                    prevBound=reduce(lambda x,y: x+y,
+                               [list(_fetchBoundLeaves(o,previousBind=True))
+                                                for o in self.optionalTrees],[])
+                    if self.optionalTrees and \
+                        reduce(lambda x,y: x+y,
+                               [list(_fetchBoundLeaves(o,previousBind=True))
+                                                for o in self.optionalTrees],[]):
+                        pass
+                    elif select :
+                        for a in select :
+                            if a in self.bindings :
+                                result[a] = self.bindings[a]
                     else :
-                        newRetval = []
-                        for optResult in optionals :
-                            # Each binding dictionary we have so far should be copied with the new values
-                            for res in retval :
-                                dct = {}
-                                # copy the content of the exisiting bindings ...
-                                dct = res.copy()
-                                # ... and extend it with the optional results
-                                for k in optResult :
-                                    if optResult[k] != None :
-                                        dct[k] = optResult[k]
-                                newRetval.append(dct)
-                        retval = newRetval
+                        result = self.bindings.copy()
+                    # Initial return block. If there is no optional processing, that is the result, in fact,
+                    # because the for cycle below will not happen
+                    retval = [result]
+                else:
+                    retval = reduce(lambda x,y: x+y,[o.returnResult(select) for o in proxies])
             return retval
-
-
+                
     def expandSubgraph(self,subTriples,pattern) :
         """
         Method used to collect the results. There are two ways to
@@ -324,7 +419,8 @@ class _SPARQLNode:
         @param r: string
         @return: returns None if no bindings occured yet, the binding otherwise
         """
-        if isinstance(r,basestring) and not isinstance(r,Identifier)  :
+        if isinstance(r,basestring) and not isinstance(r,Identifier) or \
+           isinstance(r,Variable) :
             if self.bindings[r] == None :
                 return None
             else :
@@ -336,7 +432,92 @@ class _SPARQLNode:
         else :
             return r
 
-    def expand(self,constraints) :
+    def expand(self,constraints):
+        if self.tripleStore.graph.store.batch_unification:
+            patterns=[]
+            if self.statement:
+                for statement in [self.statement]+self.rest: 
+                    (s,p,o,func) = statement
+                    searchTerms=[self._bind(term) is not None and \
+                                 self._bind(term) or term
+                                    for term in [s,p,o]]
+                    (search_s,search_p,search_o) = searchTerms#(self._bind(s),self._bind(p),self._bind(o))
+                    if self.tripleStore.graphVariable:
+                        graphName=self.bindings.get(self.tripleStore.graphVariable,
+                                                    self.tripleStore.graphVariable)
+                    elif self.tripleStore.DAWG_DATASET_COMPLIANCE and \
+                         isinstance(self.tripleStore.graph,ConjunctiveGraph):
+                        #For query-constructed datasets, match against the 'default graph' - 
+                        #the first Graph with a non-URIRef identifier (or an empty, default graph) 
+                        if isinstance(self.tripleStore.graph,ReadOnlyGraphAggregate):
+                            graphName=None
+                            for g in self.tripleStore.graph.graphs:
+                                searchRT = []
+                                if isinstance(g.identifier,BNode):
+                                    graphName=g.identifier
+                                    break
+                            if graphName is None:
+                                #No default graph was created and the active greaph 
+                                #is supposed to be the default graph
+                                #so we should have no answers
+                                continue
+                        else:
+                            #match against the default graph
+                            graphName=self.tripleStore.graph.default_context.identifier
+                    elif isinstance(self.tripleStore.graph,ConjunctiveGraph):
+                        #match all graphs
+                        graphName = Variable(BNode())
+                    else:
+                        #otherwise, the default graph is the graph queried
+                        graphName = self.tripleStore.graph.identifier
+                    patterns.append((search_s,search_p,search_o,graphName))
+                #expand at server, accumulating results
+                rt=[]
+                nonGroundPatterns=[pattern for pattern in patterns 
+                                       if not isGroundQuad(pattern)]
+                if nonGroundPatterns:
+                    #Only evaluate at the server if not all the terms are ground
+                    for rtDict in self.tripleStore.graph.store.batch_unify(patterns):
+                        if self.tripleStore.graphVariable:
+                            if self.tripleStore.DAWG_DATASET_COMPLIANCE and \
+                               isinstance(rtDict[self.tripleStore.graphVariable],BNode):
+                                #We can't match the default graph when the active
+                                #graph is set via the GRAPH expression
+                                continue
+                        rt.append(rtDict)
+                        # create a copy of the current bindings, by also adding the new ones from result of the search
+                        new_bindings = self.bindings.copy()
+                        new_bindings.update(rtDict)
+                        child = _SPARQLNode(self,new_bindings,[],self.tripleStore,expr=self.expr)
+                        self.children.append(child)
+                        assert not child.clash and child.bindings
+                        for func in constraints :
+                            try:
+                                if func(new_bindings) == False :
+                                    child.clash = True
+                                    break
+                            except TypeError:
+                                child.clash=True
+                else:
+                    #If all the patterns are ground, there is no need
+                    #to invoke server-side unification (no variables to batch unify)
+                    self.expandAtClient(constraints)
+                    return
+            if self.statement:
+                if nonGroundPatterns and len(self.children) == 0:
+                    self.clash = True
+            else:
+                for func in constraints :
+                    try:
+                        if func(self.bindings) == False :
+                            self.clash = True
+                            break
+                    except TypeError:
+                        self.clash=True
+        else:
+            self.expandAtClient(constraints)
+
+    def expandAtClient(self,constraints) :
         """
         The expansion itself. See class comments for details.
 
@@ -354,17 +535,62 @@ class _SPARQLNode:
             # put the bindings we have so far into the statement; this may add None values,
             # but that is exactly what RDFLib uses in its own search methods!
             (search_s,search_p,search_o) = (self._bind(s),self._bind(p),self._bind(o))
+            #We need to keep track of the original Graph associated with the tripleStore
+            #so we can switch back to it after changing the active graph (if we do)
+            #otherwise we will effect later evaluations which use the same tripleStore instance
+            originalGraph = None
             if self.tripleStore.graphVariable:
-                assert hasattr(self.tripleStore.graph,'quads'),\
-                  "Graph graph patterns can only be used with Graph instances with a quad method"
-                
-                searchRT = self.tripleStore.graph.quads((search_s,search_p,search_o))
+                if hasattr(self.tripleStore.graph,'quads'):
+                    if self.tripleStore.graphVariable not in self.bindings:
+                        searchRT = self.tripleStore.graph.quads((search_s,
+                                                                 search_p,
+                                                                 search_o))
+                    else:                    
+                        graphName = self.bindings[self.tripleStore.graphVariable]
+                        assert not self.tripleStore.DAWG_DATASET_COMPLIANCE or\
+                           isinstance(graphName,URIRef),\
+                           "Cannot formally return graph name solutions for the default graph!"                        
+                        unifiedGraph = Graph(self.tripleStore.graph.store,
+                                             identifier=graphName)
+                        originalGraph = self.tripleStore.graph
+                        self.tripleStore.graph = unifiedGraph
+                        searchRT = [(_s,_p,_o,unifiedGraph) 
+                                     for _s,_p,_o in \
+                                       unifiedGraph.triples((search_s,search_p,search_o))]
+                else:
+                    assert not self.tripleStore.DAWG_DATASET_COMPLIANCE or\
+                       isinstance(self.tripleStore.graph.identifier,URIRef),\
+                       "Cannot formally return graph name solutions for the default graph"
+                    searchRT = [(_s,_p,_o,self.tripleStore.graph) 
+                                  for _s,_p,_o in self.tripleStore.graph.triples((search_s,search_p,search_o))]
+            elif self.tripleStore.DAWG_DATASET_COMPLIANCE and \
+                 isinstance(self.tripleStore.graph,ConjunctiveGraph):
+                #For query-constructed datasets, match against the 'default graph' - 
+                #the first Graph with a non-URIRef identifier (or an empty, default graph) 
+                if isinstance(self.tripleStore.graph,ReadOnlyGraphAggregate):
+                        for g in self.tripleStore.graph.graphs:
+                            searchRT = []
+                            if isinstance(g.identifier,BNode):
+                                searchRT = g.triples((search_s,search_p,search_o))               
+                                break
+                else:
+                    #match against the default graph
+                    searchRT = self.tripleStore.graph.default_context.triples(
+                                       (search_s,
+                                        search_p,
+                                        search_o))
             else:
+                #otherwise, the default graph is the graph queried
                 searchRT = self.tripleStore.graph.triples((search_s,search_p,search_o))
+            if originalGraph:
+                self.tripleStore.graph = originalGraph
             for tripleOrQuad in searchRT:
-            #for (result_s,result_p,result_o) in self.tripleStore.graph.triples((search_s,search_p,search_o)) :
                 if self.tripleStore.graphVariable:
                     (result_s,result_p,result_o,parentGraph) = tripleOrQuad
+                    if isinstance(self.tripleStore.graph,ConjunctiveGraph) and \
+                       self.tripleStore.DAWG_DATASET_COMPLIANCE and isinstance(parentGraph.identifier,BNode):
+                        continue
+                    assert isinstance(parentGraph.identifier,URIRef)
                 else:
                     (result_s,result_p,result_o) = tripleOrQuad
                 # if a user defined constraint has been added, it should be checked now
@@ -373,19 +599,33 @@ class _SPARQLNode:
                     continue
                 # create a copy of the current bindings, by also adding the new ones from result of the search
                 new_bindings = self.bindings.copy()
-                if search_s == None : new_bindings[s] = result_s
-                if search_p == None : new_bindings[p] = result_p
-                if search_o == None : new_bindings[o] = result_o
+                queryTerms = [s,p,o] 
+                preClash = False
+                for searchSlot,searchTerm,result in [(search_s,s,result_s),
+                                                     (search_p,p,result_p),
+                                                     (search_o,o,result_o)]:
+                    #searchSlot is what we searched with (variables become none)
+                    #searchTerm is the term in the triple pattern
+                    #result is the unified term from the dataset
+                    if searchSlot == None :
+                        #An unknown
+                        currBound = new_bindings.get(searchTerm)
+                        if currBound is not None:
+                            if currBound != result:
+                                preClash = True
+                        else:
+                            new_bindings[searchTerm] = result
                 if self.tripleStore.graphVariable:
                     new_bindings[self.tripleStore.graphVariable] = parentGraph.identifier
-
                 # Recursion starts here: create and expand a new child
-                child = _SPARQLNode(self,new_bindings,self.rest,self.tripleStore)
-                child.expand(constraints)                
+                child = _SPARQLNode(self,new_bindings,self.rest,self.tripleStore,expr=self.expr)
+                if preClash:
+                    child.clash = True
+                else:
+                    child.expand(constraints)
                 # if the child is a clash then no use adding it to the tree, it can be forgotten
                 if self.clash == False :
                     self.children.append(child)
-
             if len(self.children) == 0 :
                 # this means that the constraints could not be met at all with this binding!!!!
                 self.clash = True
@@ -393,9 +633,12 @@ class _SPARQLNode:
             # this is if all bindings are done; the conditions (ie, global constraints) are still to be checked
             if self.bound == True and self.clash == False :
                 for func in constraints :
-                    if func(self.bindings) == False :
-                        self.clash = True
-                        break
+                    try:
+                        if func(self.bindings) == False :
+                            self.clash = True
+                            break
+                    except TypeError:
+                        self.clash=True
 
     def expandOptions(self,bindings,statements,constraints) :
         """
@@ -438,14 +681,13 @@ class _SPARQLNode:
                     if key in bindings :
                         del bindings[key]
                 bindings.update(toldBNodeLookup)
-                optTree = _SPARQLNode(None,bindings,statements,self.tripleStore)
+                optTree = _SPARQLNode(None,bindings,statements,self.tripleStore,expr=self.expr)
                 self.optionalTrees.append(optTree)
                 optTree.expand(constraints)
         else :
             for c in self.children :
                 c.expandOptions(bindings,statements,constraints)
 
-
 def _processResults(select,arr) :
     '''
     The result in an expansion node is in the form of an array of
@@ -625,6 +867,14 @@ class Query :
     def _getFullBinding(self) :
         """Retrieve the full binding, ie, an array of binding dictionaries
         """
+        if self.parent1 != None and self.parent2 != None :
+            results = self.parent1.select(None) + self.parent2.select(None)
+        else :
+            # remember: _processResult turns the expansion results (an array of dictionaries)
+            # into an array of tuples in the right, original order
+            results = self.top.returnResult(None)
+        return results
+        
         if self.parent1 != None and self.parent2 != None :
             return self.parent1._getFullBinding() + self.parent2._getFullBinding()
         else :
@@ -632,11 +882,14 @@ class Query :
             return self.top.returnResult(None)
 
     def _getAllVariables(self):
-       """Retrieve the list of all variables, to be returned"""
-       if self.parent1 and self.parent2:
-           return list2set(self.parent1._getAllVariables() + self.parent2._getAllVariables())
-       else:
-           return list2set(self.top.bindings.keys())
+        """Retrieve the list of all variables, to be returned"""
+        if self.parent1 and self.parent2:
+            return list2set(self.parent1._getAllVariables() + self.parent2._getAllVariables())
+        else:
+            maxKeys = []
+            for bound in _fetchBoundLeaves(self.top):
+                maxKeys.extend(bound.bindings.keys())
+            return list2set(maxKeys)
 
     def _orderedSelect(self,selection,orderedBy,orderDirection) :
         """
@@ -697,6 +950,7 @@ class Query :
                 return 0
         # get the full Binding sorted
         fullBinding.sort(_sortBinding)
+        
         # remember: _processResult turns the expansion results (an array of dictionaries)
         # into an array of tuples in the right, original order
         retval = _processResults(selection,fullBinding)
@@ -772,6 +1026,8 @@ class Query :
             retval = results
 
         if limit != None :
+            if limit == 0:
+                return []
             return retval[offset:limit+offset]
         elif offset > 0 :
             return retval[offset:]
