@@ -1,7 +1,5 @@
 from rdflib import sparql
-from rdflib.sparql.bison.Query import Query, Prolog
-from rdflib.sparql.bison.SPARQLEvaluate import Evaluate        
-from rdflib.sparql.bison import SPARQLParserc as SPARQLParser
+import SPARQLParserc as SPARQLParser
 
 def CreateSPARQLParser():
     return SPARQLParser.new()    
@@ -22,8 +20,16 @@ class Processor(sparql.Processor):
     def __init__(self, graph):
         self.graph = graph
 
-    def query(self, strOrQuery, initBindings={}, initNs={}, DEBUG=False):
-        assert isinstance(strOrQuery, (basestring, Query)), "%s must be a string or an rdflib.sparql.bison.Query.Query instance"%strOrQuery
+    def query(self, 
+              strOrQuery, 
+              initBindings={}, 
+              initNs={}, 
+              DEBUG=False,
+              dataSetBase=None,
+              extensionFunctions={}):
+        from rdflib.sparql.bison.Query import Query, Prolog
+        from rdflib.sparql.Algebra import TopEvaluate
+        assert isinstance(strOrQuery, (basestring, Query)),"%s must be a string or an rdflib.sparql.bison.Query.Query instance"%strOrQuery
         if isinstance(strOrQuery, basestring):
             strOrQuery = Parse(strOrQuery, DEBUG)
         if not strOrQuery.prolog:
@@ -33,4 +39,12 @@ class Processor(sparql.Processor):
             for prefix, nsInst in initNs.items():
                 if prefix not in strOrQuery.prolog.prefixBindings:
                     strOrQuery.prolog.prefixBindings[prefix] = nsInst
-        return  Evaluate(self.graph, strOrQuery, initBindings, DEBUG=DEBUG)
+                    
+        global prolog            
+        prolog = strOrQuery.prolog
+        return TopEvaluate(strOrQuery,
+                           self.graph,
+                           initBindings,
+                           DEBUG=DEBUG,
+                           dataSetBase=dataSetBase,
+                           extensionFunctions=extensionFunctions)
