@@ -23,6 +23,8 @@ from rdflib.store.REGEXMatching import REGEXTerm
 from QuadSlot import *
 Any = None
 
+EXPLAIN_INFO = False
+
 CONTEXT_COLUMN = 'context'
 ANY_TERM = ['U','B','F','V','L']
 CONTEXT_TERMS   = ['U','B','F']
@@ -65,6 +67,8 @@ class BinaryRelationPartition(object):
     """
     assertedColumnName = 'asserted'
     indexSuffix = 'Index'
+    literalTable = False
+    objectPropertyTable = False
     def __init__(self,identifier,idHash,valueHash):
         self.identifier = identifier
         self.idHash    = idHash
@@ -92,15 +96,17 @@ class BinaryRelationPartition(object):
                     self.idHash.columns[0][0])]
         return rt
 
-    def IndexManagementSQL(self,create=False):
+    def IndexManagementSQL(self,ignoreFK=False,create=False):
         idxSQLStmts = []
         for slot in POSITION_LIST:
             if self.columnNames[slot]:
                 if create:
                     idxSQLStmts.append("create INDEX %s%s on %s (%s)"%(self.columnNames[slot],self.indexSuffix,self,self.columnNames[slot]))
-                    idxSQLStmts.append("ALTER TABLE %s ADD %s"%(self,self.foreignKeySQL(slot)[0]))
+                    if not ignoreFK:
+                        idxSQLStmts.append("ALTER TABLE %s ADD %s"%(self,self.foreignKeySQL(slot)[0]))
                 else:
-                    idxSQLStmts.append("ALTER TABLE %s DROP FOREIGN KEY %s_%s_lookup"%(self,self,self.columnNames[slot]))
+                    if not ignoreFK:
+                        idxSQLStmts.append("ALTER TABLE %s DROP FOREIGN KEY %s_%s_lookup"%(self,self,self.columnNames[slot]))
                     idxSQLStmts.append("ALTER TABLE %s DROP INDEX %s%s"%(self,self.columnNames[slot],self.indexSuffix))
                 if self.termEnumerations[slot]:
                     if create:
@@ -119,9 +125,11 @@ class BinaryRelationPartition(object):
                 else:
                     if create:
                         idxSQLStmts.append("create INDEX %s%s on (%s)"%(colMD,self.indexSuffix,self,colMD))
-                        idxSQLStmts.append("ALTER TABLE %s ADD %s"%(self,self.foreignKeySQL(otherSlot)[0]))
+                        if not ignoreFK:
+                            idxSQLStmts.append("ALTER TABLE %s ADD %s"%(self,self.foreignKeySQL(otherSlot)[0]))
                     else:
-                        idxSQLStmts.append("ALTER TABLE %s DROP FOREIGN KEY %s_%s_lookup"%(self,self,colMD))
+                        if not ignoreFK:
+                            idxSQLStmts.append("ALTER TABLE %s DROP FOREIGN KEY %s_%s_lookup"%(self,self,colMD))
                         idxSQLStmts.append("drop index %s%s on %s"%(colMD,self.indexSuffix,self))
 
         return idxSQLStmts
@@ -202,6 +210,42 @@ class BinaryRelationPartition(object):
         cursor.executemany(self.singularInsertionSQLCmd,self.pendingInsertions)
         cursor.close()
         self._resetPendingInsertions()
+        
+    def viewUnionSelectExpression(self,relations_only=False):
+        """
+        Return a SQL statement which creates a view of all the RDF statements
+        from all the contributing partitions
+        """
+        rt=[]
+        if relations_only and self.objectPropertyTable:
+            return "select * from %s"%repr(self)
+        for idx in range(len(POSITION_LIST)):
+            rdfTermLabel=SlotPrefixes[idx]
+            if idx < len(self.columnNames) and self.columnNames[idx]:
+                #there is a matching column
+                rt.append(self.columnNames[idx]+' as %s'%rdfTermLabel)
+                if self.termEnumerations[idx]:
+                    #there is a corresponding term enumeration
+                    rt.append(self.columnNames[idx]+'_term as %s_term'%rdfTermLabel)
+                else:
+                    #no corresponding term enumeration (hardcoded)
+                    rt.append("'%s' as %s_term"%(self.hardCodedResultTermsTypes[idx],
+                                                 rdfTermLabel))
+            else:
+                assert self.hardCodedResultFields[idx] == RDF.type
+                rt.append("'%s' as %s"%(normalizeValue(self.hardCodedResultFields[idx],'U'),
+                                        rdfTermLabel))
+                if self.hardCodedResultTermsTypes[idx]:
+                    rt.append("'%s' as %s_term"%(self.hardCodedResultTermsTypes[idx],
+                                                 rdfTermLabel))
+        if not relations_only:
+            if self.literalTable:
+                for i in self.columnNames[-2:]:
+                    rt.append(i[0])
+            else:
+                rt.append('NULL as data_type')
+                rt.append('NULL as language')
+        return "select %s from %s"%(', '.join(rt),repr(self))        
 
     def selectContextFields(self,first):
         """
@@ -373,7 +417,7 @@ class NamedLiteralProperties(BinaryRelationPartition):
     hardCodedResultTermsTypes = {
         OBJECT    : 'L'
     }
-
+    literalTable = True
     def foreignKeySQL(self,slot):
         hash = slot == OBJECT and self.valueHash or self.idHash
         rt = ["\tCONSTRAINT %s_%s_lookup FOREIGN KEY  (%s) REFERENCES %s (%s)"%(
@@ -453,7 +497,7 @@ class NamedLiteralProperties(BinaryRelationPartition):
 
     def compileQuadToParams(self,quadSlots):
         subjSlot,predSlot,objSlot,conSlot = quadSlots
-        dTypeParam = objSlot.term.datatype and normalizeValue(objSlot.term.datatype,objSlot.termType) or None
+        dTypeParam = objSlot.term.datatype and normalizeValue(objSlot.term.datatype,'U') or None
         langParam  = objSlot.term.language and objSlot.term.language or None
         rtList = [
                     subjSlot.md5Int,
@@ -469,13 +513,31 @@ class NamedLiteralProperties(BinaryRelationPartition):
         return tuple(rtList)
 
     def extractIdentifiers(self,quadSlots):
+        """
+        Test literal data type extraction
+        >>> from rdflib import RDF
+        >>> class DummyClass:
+        ...   def __init__(self,test=False):
+        ...     self.test = test
+        ...   def updateIdentifierQueue(self,stuff): 
+        ...     if self.test: 
+        ...       term,termType = stuff[-1]
+        ...       assert termType == 'U',"Datatype's are URIs!" 
+        >>> class Tester(NamedLiteralProperties):
+        ...   def __init__(self): 
+        ...     self.idHash    = DummyClass(True)
+        ...     self.valueHash = DummyClass()
+        >>> c = Tester()
+        >>> slots = genQuadSlots([BNode(),RDF.first,Literal(1),BNode()])
+        >>> c.extractIdentifiers(slots)
+        """
         subjSlot,predSlot,objSlot,conSlot = quadSlots
         idTerms = [
                     (subjSlot.term,subjSlot.termType),
                     (predSlot.term,predSlot.termType),
                     (conSlot.term,conSlot.termType)]
         if objSlot.term.datatype:
-            idTerms.append((objSlot.term.datatype,objSlot.termType))
+            idTerms.append((objSlot.term.datatype,term2Letter(objSlot.term.datatype)))
         self.idHash.updateIdentifierQueue(idTerms)
         self.valueHash.updateIdentifierQueue([(objSlot.term,objSlot.termType)])
 
@@ -497,7 +559,7 @@ class NamedBinaryRelations(BinaryRelationPartition):
 
     hardCodedResultFields = {}
     hardCodedResultTermsTypes = {}
-
+    objectPropertyTable = True
     def compileQuadToParams(self,quadSlots):
         subjSlot,predSlot,objSlot,conSlot = quadSlots
         return (subjSlot.md5Int,
@@ -628,8 +690,11 @@ def PatternResolution(quad,cursor,BRPs,orderByTriple=True,fetchall=True,fetchCon
         query = unionQueries[0] + orderBySuffix
     else:
         query = ' union all '.join(['('+q+')' for q in unionQueries]) + orderBySuffix
-    query = query + '  # %s'%str(quad)
     try:
+        if EXPLAIN_INFO:
+            cursor.execute("EXPLAIN "+query,tuple(unionQueriesParams))
+            print query
+            from pprint import pprint;pprint(cursor.fetchall())
         cursor.execute(query,tuple(unionQueriesParams))
     except ValueError,e:
         print "## Query ##\n",query
@@ -657,7 +722,7 @@ CREATE TEMPORARY TABLE result (
     INDEX USING BTREE (context(50))
 )
 """
-CROSS_BRP_QUERY_SQL="SELECT DISTINCT %s FROM %s %s WHERE "
+CROSS_BRP_QUERY_SQL="SELECT STRAIGHT_JOIN %s FROM %s %s WHERE "
 CROSS_BRP_RESULT_QUERY_SQL="SELECT * FROM result ORDER BY context"
 DROP_RESULT_TABLE_SQL = "DROP result"
 
@@ -678,3 +743,9 @@ BRPQueryDecisionMap = {
     'U_RNTR':(NamedLiteralProperties,NamedBinaryRelations), #Could be optimized to not include NamedBinaryRelations
 }
 
+def test():
+    import doctest
+    doctest.testmod()
+
+if __name__ == '__main__':
+    test()
