@@ -20,7 +20,6 @@ import datetime
 from rdflib.sparql.graphPattern import GraphPattern
 
 thresholdDate = datetime.date(2005,01,01)
-
 rdfData = """<?xml version="1.0" encoding="UTF-8"?>
 <rdf:RDF
    xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#"
@@ -33,13 +32,17 @@ rdfData = """<?xml version="1.0" encoding="UTF-8"?>
                 <foaf:name>Alice</foaf:name>
                 <foaf:mbox rdf:resource="mailto:alice@example.com"/>
         </rdf:Description>
+        <rdf:Description>
+                <foaf:name>Bob</foaf:name>
+                <foaf:mbox rdf:resource="mailto:bob@example.org"/>
+        </rdf:Description>
 </rdf:RDF>
 """
 
 select      = []
 pattern     = GraphPattern([("?x",ns_foaf["name"],"?name")])
 optional    = []
-construct   = GraphPattern([(ns_person["Alice"],ns_vcard["FN"],"?name")])
+construct   = None
 tripleStore = None
 
 
